# Perl 5
#
# Test for Report::Porf::*
#
# Perl Open Report Framework (Porf)
#
# Ralf Peine, Sat Feb 01 08:13:01 2014
#
#------------------------------------------------------------------------------

$VERSION = "0.950";

use strict;
use warnings;

$|=1;

use Time::HiRes;

use Report::Porf::Framework;
use Report::Porf::Util;
use Report::Porf::Table::Simple;
use Report::Porf::Table::Simple::HtmlReportConfigurator;
use Report::Porf::Table::Simple::TextReportConfigurator;
use Report::Porf::Table::Simple::CsvReportConfigurator;

#--------------------------------------------------------------------------------
#
#  Run
#
#--------------------------------------------------------------------------------

use Test::More tests => 49;
use Test::Exception;

T010_CreateInstances();
T100_Align();
T110_InterpreteValueOptions();
T020_Verbose();
T300_TextExport();
T310_HtmlExport();
T320_CsvExport();
T400_AutoReport();
T800_UseFramework();
T900_ExceptionsDuringSetup();

exit 0;

#--------------------------------------------------------------------------------
#
#  Create Test Data
#
#--------------------------------------------------------------------------------

# --- create persons as array, first entry is constant, following not -----------
sub create_persons_as_array {
	my $max_entries = shift;

	my $start_time = HiresActualTime();
	my @rows;
	
	foreach my $l (1..$max_entries) {
		my $time = HiresDiffTime($start_time, HiresActualTime());
		$time = '8e-06' if $l == 1;
		my @data = (
			$l,
			"Vorname $l",
			"Name $l",
			($l/$max_entries)*100,
			$time,
		);

		push (@rows, \@data);	
	}

	return \@rows;
}

#--------------------------------------------------------------------------------
#
#  Helper subs
#
#--------------------------------------------------------------------------------

# --- get hiresolution time, uses Time::HiRes ---
sub HiresActualTime {
	return [Time::HiRes::gettimeofday];
}

# --- get difference from now to $start_time in hiresolution time, uses Time::HiRes ---
sub HiresDiffTime {
	my $start_time = shift;
	my $end_time   = shift; # actual time, if omitted
	return Time::HiRes::tv_interval ($start_time, $end_time); 
}

#--------------------------------------------------------------------------------
#
#  Configure tables
#
#--------------------------------------------------------------------------------

sub create_simple_test_table_columns_for_array_data {
	my $t_o = shift; # $test_object
	
	# --- Configure table ------------------------------------------------------------
	#
	# Order of calls gives order of columns in table
	$t_o->ConfigureColumn(-header => 'Count',      -align => 'Center', -value_indexed => 0 );
	$t_o->ConfigureColumn(-header => 'TimeStamp', -w => 10, -a => 'R', -val_idx       => 4 );
	$t_o->ConfigureColumn(-h      => 'Age',       -w =>  7, -a => 'C', -vi            => 3 );
	$t_o->ConfigureColumn(-h      => 'Prename',   -w => 11, -a => 'l', -value => sub { return $_[0]->[1]; } );
	$t_o->ConfigureColumn(-h      => 'Surname',   -width =>  8,        -v     =>             '$_[0]->[2]'   );
	
	# --- Configure table end --------------------------------------------------------

	$t_o->ConfigureComplete();
}

sub create_simple_test_table_columns_for_hash_data {
	my $t_o = shift; # $test_object
	
	# --- Configure table ------------------------------------------------------------
	#
	# Order of calls gives order of columns in table
	$t_o->ConfigureColumn(-h      => 'Age',       -w =>  7, -a => 'C', -value_named => 'Age' );
	$t_o->ConfigureColumn(-h      => 'Prename',   -w => 11, -a => 'l', -val_nam     => 'Prename');
	$t_o->ConfigureColumn(-h      => 'Surname',   -width =>  8,        -vn          => 'Surname');
	
	# --- Configure table end --------------------------------------------------------

	$t_o->ConfigureComplete();
}

#------------------------------------------------------------------------------
#
#  Tests
#
#------------------------------------------------------------------------------

# --- Test Creation --------------------------------------------------------
sub T010_CreateInstances {

	my $test_object;
	
	lives_and {
		ok ($test_object = Report::Porf::Table::Simple->new(),
			'create instance');
	} "create instance does not die!";

	is ($test_object->GetMaxColWidth(),   0, 'initial MaxColWidth');
	is ($test_object->GetMaxColumnIdx(), -2, 'initial MaxColumnIdx');
}

# --- Test Verbose --------------------------------------------------------
sub T020_Verbose {

	my $test_object = Report::Porf::Table::Simple->new();

	$test_object->SetVerbose(0);
	is (Verbose($test_object,  ), 0, 'verbose( ) 0 0');
	is (Verbose($test_object, 1), 0, 'verbose(1) 0 0');
	is (Verbose($test_object, 8), 0, 'verbose(8) 0 0');

	$test_object->SetVerbose(2);
	is (Verbose($test_object,  ), 2, 'verbose( ) 2 2');
	is (Verbose($test_object, 1), 2, 'verbose(1) 2 0');
	is (Verbose($test_object, 8), 0, 'verbose(8) 2 8');
}

# --- Test Align --------------------------------------------------------
sub T100_Align {
	# --- left => L -----------------------------------------------------------
	is (InterpreteAlignment('left'  ), 'Left', "alignment 'left'");
	is (InterpreteAlignment('l'     ), 'Left', "alignment 'l'");
	is (InterpreteAlignment('  L'   ), 'Left', "alignment '  L'");
	is (InterpreteAlignment('lEfT'  ), 'Left', "alignment 'LeFt'");
	is (InterpreteAlignment('lEfT  '), 'Left', "alignment 'LeFt  '");

	# --- center => C -----------------------------------------------------------
	is (InterpreteAlignment('center'  ), 'Center', "alignment 'center'");
	is (InterpreteAlignment('c'       ), 'Center', "alignment 'c'");
	is (InterpreteAlignment(' C '     ), 'Center', "alignment ' C '");
	is (InterpreteAlignment('cEnTeR'  ), 'Center', "alignment 'cEnTeR'");
	is (InterpreteAlignment(' cEnTeR '), 'Center', "alignment ' cEnTeR '");

	# --- right => R -----------------------------------------------------------
	is (InterpreteAlignment('right'  ), 'Right', "alignment 'right'");
	is (InterpreteAlignment('r'      ), 'Right', "alignment 'r'");
	is (InterpreteAlignment('  R'    ), 'Right', "alignment '  R'");
	is (InterpreteAlignment('RiGHt'  ), 'Right', "alignment 'RiGHt'");
	is (InterpreteAlignment('  RiGHt'), 'Right', "alignment '  RiGHt'");

	# --- bla => dies -----------------------------------------------------
	throws_ok {
	    InterpreteAlignment(' bla     ');
	}
		qr/cannot interprete alignment/i,
			"configure align by unallowed value 'bla'";
}

# --- Test the interpreter for value options ----------------------------
sub T110_InterpreteValueOptions {

    is (InterpreteValueOptions ({ -value => '$bla;'}), '$bla;', '-value => $bla;');
    is (InterpreteValueOptions ({ -val   => '$bla;'}), '$bla;', '-val   => $bla;');
    is (InterpreteValueOptions ({ -v     => '$bla;'}), '$bla;', '-v     => $bla;');

    is (InterpreteValueOptions ({ -value_indexed => 0 }), '$_[0]->[0]', '-value_indexed => 0');
    is (InterpreteValueOptions ({ -val_idx       => 1 }), '$_[0]->[1]', '-val_idx       => 1');
    is (InterpreteValueOptions ({ -vi            => 2 }), '$_[0]->[2]', '-vi            => 2');

    is (InterpreteValueOptions ({ -value_named   => 'x'   }), '$_[0]->{x}',   "-value_named => 'x'  ");
    is (InterpreteValueOptions ({ -val_nam       => 'x_y' }), '$_[0]->{x_y}', "-val_nam     => 'x_y'");
    is (InterpreteValueOptions ({ -vn            => '_'   }), '$_[0]->{_}',   "-vi          => '_'  ");

    is (InterpreteValueOptions ({ -value_object  => ' maximal () ' }), '$_[0]->maximal()', "-value_object => ' maximal () '");
    is (InterpreteValueOptions ({ -val_obj       => 'Max_1A()'     }), '$_[0]->Max_1A()' , "-val_obj      => 'Max_1A()'    ");
    is (InterpreteValueOptions ({ -vo            => 'min'          }), '$_[0]->min()'    , "-vo           => 'min'         ");
}

# --- Test text export --------------------------------------------------------
sub T300_TextExport {

	my $text_report_configurator = Report::Porf::Table::Simple::TextReportConfigurator->new();
	my $test_object = Report::Porf::Table::Simple->new();
	# $test_object->SetVerbose(3);
	# $test_object->SetVerbose(2);
	
	# --- Test -------------------------------------------------------	
	lives_and {
		ok ($text_report_configurator->ConfigureReport($test_object), 'configure text report');
	} "configure(text) does not die!";

	# --- Test -------------------------------------------------------	

	# --- Test: prepare ---
	my $person_rows = create_persons_as_array(10);

	create_simple_test_table_columns_for_array_data($test_object);

	# --- Test: call and validate ---
	is ($test_object->GetRowOutput($person_rows->[0]),
		"|      1     |      8e-06 |    10   | Vorname 1   | Name 1   |\n",
		'create one row of output');

	# --- Test -------------------------------------------------------	
	# --- Test: prepare ---
	# --- Test: call ---
	# --- Test validation ---
}

# --- Test Html export --------------------------------------------------------
sub T310_HtmlExport {

	my $html_report_configurator = Report::Porf::Table::Simple::HtmlReportConfigurator->new();
	my $test_object = Report::Porf::Table::Simple->new();
	# $test_object->SetVerbose(3);
	# $test_object->SetVerbose(2);
	$html_report_configurator->SetVerbose(3);
	
	# --- Test -------------------------------------------------------	
	lives_and {
		ok ($html_report_configurator->ConfigureReport($test_object), 'configure text report');
	} "configure(text) does not die!";

	# --- Test -------------------------------------------------------	

	# --- Test: prepare ---
	my $person_rows = create_persons_as_array(10);

	create_simple_test_table_columns_for_array_data($test_object);

	# --- Test: call and validate ---
	is ($test_object->GetRowOutput($person_rows->[0]),
		'<tr><td  align="center">1</td><td  align="right">8e-06</td><td  align="center">10</td>'
			.'<td  align="left">Vorname 1</td><td  align="left">Name 1</td></tr>'
				."\n",
		'create one row of output');

	# --- Test -------------------------------------------------------	

	# --- Test: prepare ---
	$test_object = Report::Porf::Framework::Get()->CreateReport('html');
	# $test_object->SetVerbose(3);

	my $person = {
		Prename => 'Ralf',
		Age => 48,
		Surname => "<Title>&<Name>&...",
	};
	
	create_simple_test_table_columns_for_hash_data($test_object);

	# --- Test: call and validate ---
	is ($test_object->GetRowOutput($person),
		'<tr bgcolor="#FFFFFF"><td  align="center">48</td><td  align="left">Ralf</td>'
            .'<td  align="left">&lt;Title&gt;&amp;&lt;Name&gt;&amp;...</td></tr>'
				."\n",
		'create one row of output with special chars escaped');

	# --- Test -------------------------------------------------------	
	# --- Test: prepare ---
	# --- Test: call ---
	# --- Test validation ---
}

# --- Test csv export --------------------------------------------------------
sub T320_CsvExport {

	my $csv_report_configurator = Report::Porf::Table::Simple::CsvReportConfigurator->new();
	my $test_object = Report::Porf::Table::Simple->new();
	# $test_object->SetVerbose(3);
	# $test_object->SetVerbose(2);
	
	# --- Test -------------------------------------------------------	
	lives_and {
		ok ($csv_report_configurator->ConfigureReport($test_object), 'configure csv report');
	} "configure(csv) does not die!";

	# --- Test -------------------------------------------------------	

	# --- Test: prepare ---
	my $person_rows = create_persons_as_array(10);

	create_simple_test_table_columns_for_array_data($test_object);

	# --- Test: call and validate ---
	is ($test_object->GetRowOutput($person_rows->[0]),
		"1,8e-06,10,Vorname 1,Name 1\n",
		'create one row of output');

	# --- Test -------------------------------------------------------	
	# --- Test: prepare ---
	# --- Test: call ---
	# --- Test validation ---
}

# --- Auto Report ----------------------------------------------

sub T400_AutoReport {

    my $sfh = new FileHandle('>/dev/null');
    my $data_rows = [{Vorname    => 'Ralf',
		      Nachname   => 'Peine',
		      Geburtstag => '29.12.1965',
		      Wohnort    => 'Bocholt'
		     },
		     {bla => 'bla'},
		     {bla => 'bla'},
		     {bla => 'bla'},
		     {Vorname    => 'Ralf',
		      Nachname   => 'Peine',
		      Geburtstag => '29.12.1965',
		      Wohnort    => 'Bocholt'
		     },
		     {bla => 'bla'},
		     {bla => 'bla'},
		     {bla => 'bla'},
		     {bla => 'bla'},
		     {bla => 'bla'},
		     {bla => 'bla'},
		     {Vorname    => 'Ralf',
		      Nachname   => 'Peine',
		      Geburtstag => '29.12.1965',
		      Wohnort    => 'Bocholt'
		     },
	];
		      

    # --- Test -------------------------------------------------------	
    lives_and {
	Report::Porf::Framework::AutoReport(
	    [{Vorname    => 'Ralf',
	      Nachname   => 'Peine',
	      Geburtstag => '29.12.1965',
	      Wohnort    => 'Bocholt'
	     }],
	    -file => $sfh);
    } "Auto Report for one element does not die";
    
    # --- Test -------------------------------------------------------	
    my $result;
    lives_and {
	Report::Porf::Framework::AutoReport(
	    $data_rows,
	    -file     => $sfh);
    } "Auto Report gives out max 10 elements per default does not die";

    # --- Test -------------------------------------------------------	
    lives_and {
	Report::Porf::Framework::AutoReport(
	    $data_rows,
	    -max_rows => 4,
	    -file     => $sfh);
    } "Auto Report gives out max 4 elements does not die";
    
    # Report::Porf::Framework::AutoReport($data_rows);

    # diag ('Auto Report complete.');
}

# --- Test framework class -------------------------------------------
sub T800_UseFramework {

	my $test_object;

	# --- Test Text -------------------------------------------------------	

	# --- Test: prepare ---
	$test_object = Report::Porf::Framework::Get();
	my $person_rows = create_persons_as_array(10);

	# --- Test: call ---
	my $text_report = $test_object->CreateReport('text');

	# --- Test: validate ---
	create_simple_test_table_columns_for_array_data($text_report);
	is ($text_report->GetRowOutput($person_rows->[0]),
		"|      1     |      8e-06 |    10   | Vorname 1   | Name 1   |\n",
		'create text report by default framework');

	# --- Test Html -------------------------------------------------------	

	# --- Test: prepare ---
	$test_object = Report::Porf::Framework::Create(-name => 'T800');
	$person_rows = create_persons_as_array(10);

	# --- Test: call ---
	my $html_report = $test_object->CreateReport('html');

	# --- Test: validate ---
	create_simple_test_table_columns_for_array_data($html_report);
	is ($html_report->GetRowOutput($person_rows->[0]),
		'<tr bgcolor="#FFFFFF"><td  align="center">1</td>'
	    .'<td  align="right">8e-06</td><td  align="center">10</td>'
		.'<td  align="left">Vorname 1</td><td  align="left">Name 1</td></tr>'
		."\n",
		'create html report by default framework');

	# --- Test Csv -------------------------------------------------------	

	# --- Test: prepare ---
	$test_object = Report::Porf::Framework::Get();
	$person_rows = create_persons_as_array(10);

	# --- Test: call ---
	my $csv_report = $test_object->CreateReport('csv');

	# --- Test: validate ---
	create_simple_test_table_columns_for_array_data($csv_report);
	is ($csv_report->GetRowOutput($person_rows->[0]),
		"1,8e-06,10,Vorname 1,Name 1\n",
		'create csv report by default framework');
}


# --- Test exceptions, that can occur during setup phase--------------
sub T900_ExceptionsDuringSetup {

	my $text_report_configurator = Report::Porf::Table::Simple::TextReportConfigurator->new();
	my $test_object = Report::Porf::Table::Simple->new();
	# $test_object->SetVerbose(3);
	# $test_object->SetVerbose(2);
	
	# --- Test -------------------------------------------------------	
	
	# --- Test: prepare ---
	$text_report_configurator->ConfigureReport($test_object);
	
	# --- Test: call and validate ---
	throws_ok {
		$test_object->ConfigureColumn(
			-h     => 'Age',
			-w     => sub { return 7; },
			-value => sub { return $_[0]->[3]; } );
	}
		qr/currently only strings are supported/i,
			"configure width by reference";

	# --- Test -------------------------------------------------------	

	# --- Test: call and validate ---
	throws_ok {
		$test_object->ConfigureColumn(
			-h     => 'Age',
			-a     => sub { return 'L'; },
			-w     => 7,
			-value => sub { return $_[0]->[3]; } );
	}
		qr/currently only strings are supported/i,
			"configure align by reference";

	# --- Test -------------------------------------------------------	

	# --- Test: call and validate ---
	throws_ok {
		$test_object->ConfigureColumn(
			-h     => 'Age',
			-a     => 'bla',
			-w     => 7,
			-value => sub { return $_[0]->[3]; } );
	}
		qr/cannot interprete alignment/i,
			"configure align by unallowed value 'bla'";

	# --- Test -------------------------------------------------------	
	# --- Test: prepare ---
	# --- Test: call ---
	# --- Test validation ---
}
