# Perl 
#
# Class Report::Porf::Framework
#
# Perl Open Report Framework (Porf)
#
# Framework to create/configure Reports for any output format.
#
# Ralf Peine, Fri Mar 28 06:43:50 2014
#
# More documentation at the end of file
#------------------------------------------------------------------------------

$VERSION = "0.953";

#------------------------------------------------------------------------------
#
# Example list with 10 lines (html)
# 
# <html>
# <table border='1' rules='groups'>
#                                         
# <thead>
# <tr><th>Coun</th><th>TimeStamp </th><th>Age    </th><th>Prename    </th><th>Surname </th></tr>
# </thead>
#                                         
# <tr><td>1   </td><td>0.000433  </td><td>10     </td><td>Vorname 1  </td><td>Name 1  </td></tr>
# <tr><td>2   </td><td>0.000638  </td><td>20     </td><td>Vorname 2  </td><td>Name 2  </td></tr>
# <tr><td>3   </td><td>0.000781  </td><td>30     </td><td>Vorname 3  </td><td>Name 3  </td></tr>
# <tr><td>4   </td><td>0.000922  </td><td>40     </td><td>Vorname 4  </td><td>Name 4  </td></tr>
# <tr><td>5   </td><td>0.001062  </td><td>50     </td><td>Vorname 5  </td><td>Name 5  </td></tr>
# <tr><td>6   </td><td>0.001203  </td><td>60     </td><td>Vorname 6  </td><td>Name 6  </td></tr>
# <tr><td>7   </td><td>0.001346  </td><td>70     </td><td>Vorname 7  </td><td>Name 7  </td></tr>
# <tr><td>8   </td><td>0.001486  </td><td>80     </td><td>Vorname 8  </td><td>Name 8  </td></tr>
# <tr><td>9   </td><td>0.001631  </td><td>90     </td><td>Vorname 9  </td><td>Name 9  </td></tr>
# <tr><td>10  </td><td>0.001773  </td><td>100    </td><td>Vorname 10 </td><td>Name 10 </td></tr>
#                                         
# </table><p/>
# # Time needed for export of 10 data lines: 0.002013
# 
# </html>
# 

use strict;
use warnings;

#--------------------------------------------------------------------------------
#
#  Report::Porf::Framework
#
#--------------------------------------------------------------------------------

package Report::Porf::Framework;

use Carp;

use Report::Porf::Table::Simple;
use Report::Porf::Table::Simple::AutoColumnConfigurator;

# only for default creation
use Report::Porf::Table::Simple::HtmlReportConfigurator;
use Report::Porf::Table::Simple::TextReportConfigurator;
use Report::Porf::Table::Simple::CsvReportConfigurator;

our %Store; # of named frameworks

our $DefaultFramework = '';

#--------------------------------------------------------------------------------
#
#  Creation / Filling Of Instances
#
#--------------------------------------------------------------------------------

# --- new Instance, Do NOT call direct!!! -----------------
sub _new
{
    my $caller = $_[0];
    my $class  = ref($caller) || $caller;
    
    # let the class go
    my $self = {};
    bless $self, $class;

    $self->{Configurators} = {};

    return $self;
}

# --- CreateInstance --------------------------------------
sub Create {
        my %options = @_ if scalar @_;

        my $framework = Report::Porf::Framework->_new();
        $framework->UseDefaultConfiguratorCreators();

        $framework->SetName($options{-name}) if $options{-name};
        $framework->SetDescription($options{-description}) if $options{-description};
	$framework->SetDefaultFormat('text');
	$framework->SetMaxRows(10);

	foreach my $key(sort(keys(%options))) {
	    if ($key =~ /^\-Create(\w+)ConfiguratorAction$/) {
             my $format = $1;
             $framework->SetConfiguratorAction($format => $options{$key});
	    }
	}

        $framework->Store();

        return $framework;
}

# --- Activate the default configurators for every known output format -------------
sub UseDefaultConfiguratorCreators {
        my ($self,        # instance_ref
        ) = @_;

        $self->SetConfiguratorAction(HTML => sub {
                        my $configurator = Report::Porf::Table::Simple::HtmlReportConfigurator->new();
                        $configurator->SetAlternateRowColors('#DDDDDD', '#FFFFFF');
                        return $configurator;
                });
        $self->SetConfiguratorAction
	    (Text => sub { return Report::Porf::Table::Simple::TextReportConfigurator->new();});
        $self->SetConfiguratorAction
	    (CSV => sub { return Report::Porf::Table::Simple::CsvReportConfigurator->new();});
}

#--------------------------------------------------------------------------------
#
#  Attributes
#
#--------------------------------------------------------------------------------

# --- Name ----------------------------------------------------------------------------

sub SetName {
        my ($self,        # instance_ref
                $value        # value to set
        ) = @_;

        if ($self->{Name}) {
                if ($self->{Name} ne $value) {
                        die "Name of framework cannot be changed!";
                }
        }
        $self->{Name} = $value;
}

sub GetName {
        my ($self,        # instance_ref
        ) = @_;
        
        return $self->{Name};
}

# --- Description ----------------------------------------------------------------------------

sub SetDescription {
        my ($self,        # instance_ref
                $value        # value to set
        ) = @_;

        $self->{Description} = $value;
}

sub GetDescription {
        my ($self,        # instance_ref
        ) = @_;
        
        return $self->{Description};
}

# --- Default Format ----------------------------------------------------------------------------

sub SetDefaultFormat {
        my ($self,        # instance_ref
            $value        # value to set
        ) = @_;

        $self->{DefaultFormat} = $value;
}

sub GetDefaultFormat {
        my ($self,        # instance_ref
        ) = @_;
        
        return $self->{DefaultFormat};
}

# --- MaxRows ----------------------------------------------------------------------------

sub SetMaxRows {
        my ($self,        # instance_ref
                $value        # value to set
        ) = @_;

        $self->{MaxRows} = $value;
}

sub GetMaxRows {
        my ($self,        # instance_ref
        ) = @_;
        
        return $self->{MaxRows};
}

# --- Configurator ---------------------------------------------------------------

sub SetConfiguratorAction {
        my ($self,        # instance_ref
            $format,      # (html/text/csv)
            $action       # value to set
        ) = @_;
        
	die "Format not set" unless $format;
	die "No action given" unless ref ($action) eq 'CODE';

        $self->{Configurators}->{lc($format)} = $action;
}

sub GetConfiguratorAction {
        my ($self,        # instance_ref
            $format,      # (html/text/csv)
        ) = @_;
        
	die "Format not set" unless $format;

        return $self->{Configurators}->{lc($format)};
}

#--------------------------------------------------------------------------------
#
#  Methods
#
#--------------------------------------------------------------------------------

# --- Get framework --- Creates default, if not existing ------------------------
sub Get {
        my ($name        # Optional: name of framework
        ) = @_;

        my $framework;

        $name = $DefaultFramework unless $name;

        unless ($name) {
                $name = '-default';
                $framework = $Store{$name};
                $framework = Create(
                        -name        => $name,
                        -description => 'Automatic created default framework')
                        ->Store()
                                unless $framework;
        }

        $framework = $Store{$name};

        confess "No report framework with name '$name' stored"
                unless $framework;

        return $framework;
}

# --- Store Framework in internal %Store ------------------------------------------

sub Store {
        my ($self,        # instance_ref
        ) = @_;

        my $name = $self->GetName();
        
        confess "cannot store unnamed framework" unless $name;

        $Store{$name} = $self;
}

# --- Define the default framework to use --------------------------------------------------
sub SetDefaultFramework {
        my ($name,        # instance_ref
            ) = @_;

        Get($name); # dies if not existing 
        my $old = $DefaultFramework;
        $DefaultFramework = $name;

        return $old;
}

# --- Create Report Configurator --------------------------------------------------

sub CreateReportConfigurator {
        my ($self,        # instance_ref
            $format       # format of report
        ) = @_;

        my $action = $self->GetConfiguratorAction(lc($format));

	die "don't know how to configure report for format '$format'"
	    unless $action;

	return $action->();
}

# --- Create Report ---------------------------------------------------------------

sub CreateReport {
        my ($self,        # instance_ref
            $format       # format of report
        ) = @_;

	$format = $self->SetDefaultFormat('text') unless $format;

        my $report_configurator = $self->CreateReportConfigurator($format);
        return $report_configurator->CreateAndConfigureReport();
}

# --- auto report -----------------------------------------------------------------

sub AutoReport {
    my ($list_ref,    # what to print out
	%args         # named args
	) = @_;

    return unless scalar @$list_ref;

    my $file_item = $args{-file} if $args{-file};
    my $max_rows  = $args{-max_rows} if defined $args{-max_rows};

    my $report_framework = Report::Porf::Framework::Get();
    my $configurator     = Report::Porf::Table::Simple::AutoColumnConfigurator->new();
    my $report           = $configurator->CreateReport($list_ref, $report_framework);

    $max_rows = $report_framework->GetMaxRows() unless defined $max_rows;

    # only max rows without file item set (print not too many rows to stdout)
    unless ($file_item) {
	if (scalar @$list_ref > $max_rows) {
	    $max_rows--;
	    my @rows = @{$list_ref}[0..$max_rows];
	    $list_ref = \@rows
	}
    }

    $report->WriteAll($list_ref, $file_item);
}

1;

__END__

=head1 NAME

Report::Porf::Framework

Framework to create/configure Reports for any output format.

Part of Perl Open Report Framework (Porf).

=head1 VERSION

This documentation refers to version 0.930 of Report::Porf::Framework

=head1 SYNOPSIS

=head2 Structure of a Report

  Report
    Table
      Line (Conatining data of data row)
        Cell
  
  *============+============+============*  # Bold separator line
  |   Prename  |   Surname  |     Age    |  # The header line
  *------------+------------+------------*  # Separator line
  | Vorname 1  | <a cell>   | 7.69230769 |  # A data line with <cell>s
  | Vorname 2  | Name 2     | 15.3846153 |
  | Vorname 3  | Name 3     | 23.0769230 |
  | Vorname 4  | Name 4     | 30.7692307 |
  *============+============+============*

=head2 Create Report

  my $report_frame_work = Report::Porf::Framework::Get();
  my $report            = $report_frame_work->CreateReport($format);

  # $report->SetVerbose(3); # uncomment to see infos about configuring phase

Current supported formats:

  HTML
  Text
  Csv

=head2 Configure Report

After creation a report has to be configured. 

Call C<ConfigureColumn(%options)> to configure a report. Following
options are available:

=head3 Layout

  -header  -h   constant: Text
  -align   -a   constant: (left|center|right)
                          (l   |   c  |    r)
  -width   -w   constant: integer
  -format  -f   constant: string for sprintf
  -color   -c   constant / sub {...}

The sub {...} makes conditional coloring easy possible.

As next, access to the value has to be defined. There are 4 alternatives
to get the value of a cell depending of type (array, hash, object).

=head3 GetValue Alternative 1 --- ARRAY

  my $prename = 1;
  my $surname = 2;
  my $age     = 3;

  $report->ConfigureColumn(-header => 'Prename', -value_indexed => $prename ); # long
  $report->ConfCol        (-h      => 'Surname', -val_idx       => $surname ); # short
  $report->CC             (-h      => 'Age',     -vi            => $age     ); # minimal

=head3 GetValue Alternative 2 --- HASH

  $report->ConfigureColumn(-header => 'Prename', -value_named => 'Prename' ); # long
  $report->ConfCol        (-h      => 'Surname', -val_nam     => 'Surname' ); # short
  $report->CC             (-h      => 'Age',     -vn          => 'Age'     ); # minimal

=head3 GetValue Alternative 3 --- OBJECT

  $report->ConfigureColumn(-header => 'Prename', -value_object => 'GetPrename()'); # long
  $report->ConfCol        (-h      => 'Surname', -val_obj      => 'GetSurname()'); # short
  $report->CC             (-h      => 'Age',     -vo           => 'GetAge()'    ); # minimal

=head3 GetValue Alternative 4 --- Free 

  $report->ConfigureColumn(-h => 'Prename',  -value =>    '"Dr. " . $_[0]->{Surname}'    );
  $report->ConfCol        (-h => 'Surname',    -val => sub { return $_[0]->{Prename}; }; );
  $report->CC             (-h => 'Age (Months)', -v =>     '(12.0 * $_[0]->GetAge())'    );

=head2 When All Columns Are Configured

  $report->ConfigureComplete(); 

=head2 Write Table Out Into File

  $report->WriteAll($person_rows, $out_file_name);

You can also put out single rows or single cells or start actions to do that.
[Needs to be explained more]

In "Report/Porf/examples" subdir you can find more examples.

=head1 Details

Here are the details for those, who want to modify an existing or
create a new ReportConfigurator. It's actually not complete. See
*ReportConfigurator.pm for more.

=head2 Report Attributes

There a following attributes of report, that can used by Get*/Set*

  FileStart
    PageStart
      TableStart
        *============+====  # BoldSeparatorLine
        |   Prename  | ...  # HeaderRowStart HeaderStart <HeaderText> HeaderEnd ... HeaderRowEnd
        *------------+----  # SeparatorLine
        | Vorname 1  | ...  # RowStart       CellStart   <CellValue>  CellEnd   ... RowEnd
        | ...        | ...  # ...
        *============+====  # BoldSeparatorLine
      TableEnd
    PageEnd
  FileEnd

To be continued...

=head1 LICENSE AND COPYRIGHT

Copyright (c) 2013 by Ralf Peine, Germany.  All rights reserved.

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.6.0 or,
at your option, any later version of Perl 5 you may have available.

=head1 DISCLAIMER OF WARRANTY

This library is distributed in the hope that it will be useful,
but without any warranty; without even the implied warranty of
merchantability or fitness for a particular purpose.

=cut
